/*
 *  GOL - Game of life simulation software
 *  Copyright (C) 1996 Michael Raymond Brown
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Michael Raymond Brown
 *  EMail:  michael@arrakeen.demon.co.uk
 *  WWW:    http://www.arrakeen.demon.co.uk/
 *  Address:  Michael Brown
 *            Kia Ora
 *            Brook Street
 *            Great Bromley
 *            Nr Colchester
 *            Essex
 *            CO7 7HX
 *            England
 */


#ifndef _tWidget_h
#define _tWidget_h

#ifdef __GNUC__
#pragma interface
#endif


#include "Core.h"
#include <Xm/Xm.h>

class xwHandler
   {public:
	virtual ~xwHandler();

	enum {button1=0x1, button2=0x2, button3=0x4};

	virtual void onMenu(class xwWindow*, int index);

	virtual void onArm(class xwWidget*, int index);		// XmNarmCallback
	virtual void onDisarm(class xwWidget*, int index);	// XmNdisarmCallback

	virtual void onClick(class xwWidget*, int index);   // XmNactivateCallback
	virtual void onChange(class xwWidget*, int index);  // XmNvalueChangedCallback

	virtual void mouseDown(class xwWidget*, int index, int button, int x, int y);
	virtual void mouseMove(class xwWidget*, int index, int button, int x, int y);
	virtual void   mouseUp(class xwWidget*, int index, int button, int x, int y);

	virtual void onExpose(class xwWidget*, int index, int x, int y, int w, int h);
	virtual void onIdle(class xwWidget*, int index);

	virtual void onResize(class xwWidget*, int index);
	virtual void onDestroy(class xwWindow*, int index); // xwWindow only
   };

#define xwLEFT		1
#define xwRIGHT		2
#define xwTOP		4
#define xwBOTTOM	8

class xwWidget : public xwCore
   {
    private:
	static void cbArm(Widget, XtPointer, XtPointer);
	static void cbDisarm(Widget, XtPointer, XtPointer);
	static void cbChange(Widget, XtPointer, XtPointer);
	static void cbClick(Widget, XtPointer, XtPointer);
	static void cbExpose(Widget, XtPointer, XtPointer);
	static void cbResize(Widget, XtPointer, XtPointer);
	static void cbMotion(Widget, XtPointer, XEvent*, Boolean*);

    protected:
	Widget      widget;
	xwWidget*   parent;
	xwHandler* handler;
	int    handler_arg;
	class xwApp* parent_app;

	int	    button;

	unsigned int default_methods;
	unsigned int default_events ;

	enum {evnt_Click  =0x1
	     ,evnt_Change =0x2
	     ,evnt_Expose =0x4
	     ,evnt_Resize =0x8
	     ,evnt_Motion =0x10
	     ,evnt_Arm    =0x20
	     ,evnt_Disarm =0x40
	     };
	enum {meth_Caption=0x1
	     ,meth_State  =0x2
	     ,meth_Size   =0x4
	     ,meth_Value  =0x8
	     ,meth_Close  =0x10
	     };

	void default_callbacks(unsigned int envt=0, unsigned int meth=0);
	xwWidget();

	inline void   linkParent(xwWidget* p) {((xwCore*)parent)->addWidget(this); }
	inline void unlinkParent(xwWidget* p) {((xwCore*)parent)->removeWidget(this); }

    public:
	~xwWidget();


	inline Widget getWidget()
	    {return widget; }


	inline void setHandler(xwHandler* cb)
	   {handler = cb; }
	inline void setHandler(xwHandler* cb, int index)
	   {handler = cb; handler_arg = index; }
	inline xwHandler* getHandler()
	   {return handler; }

	inline setIndex(int index)
	   {handler_arg=index; }
	inline int getIndex()
	   {return handler_arg; }


	virtual void setCaption(const char*);
	virtual void getCaption(char*);

	virtual void setValue(int onoff);
	virtual int  getValue(void);

	virtual void setSize(int width, int height);
	virtual void getSize(int* width, int* height);

	virtual void setEnabled(int v);
	virtual int  getEnabled();

	virtual void move(int x, int y);

	class xwApp* getApp();
   };

#endif

